// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function problem = sopi_addObjective(problem, fun)
   // Adds an objective function to an optimisation problem.
   //
   // Calling Sequence
   //    problem = sopi_addObjective(problem, fun)
   //
   // Parameters
   //    problem  : the optimisation problem (sopiPro)
   //    fun      : the objective function (sopiVar)
   //
   // Description
   //    This routine adds an objective function to an optimisation problem.
   //    In particular, it checks the class of the objective function in order to
   //    determine (later) the type of problem. In addition, when possible,
   //    it extracts canonical information from the objective function (e.g. c
   //    matrix in linear problems)
   //
   problem.objective = fun
   if fun.isLinear then
      problem.objType   = 1
      [c,q]             = sopi_extractAffineForm(fun, problem)
      problem.fun       = struct('c',c','q',q)
   elseif fun.isPoly & fun.polyOrder == 2 & fun.isSmooth then
      problem.objType   = 2
      [H,c,q]           = sopi_extractQuadForm(fun, problem)
      problem.fun       = struct('H',H,'c',c','q',q)
      if isscalar(H)
         ev = H
      else
         ev = eigs(H,[],1,'SA')
      end
      if ev > 0 
         // Problem is convex
         problem.objType = 20
      elseif ev >= 0
         problem.objType = 21
      elseif ev < 0
         // Problem is concave
         problem.objType = 22
      elseif ev <= 0
         problem.objType = 23
      else
         // Problem is indefinite
         problem.objType = 24
      end
//   elseif fun.isPWA & fun.isConvex then
//      problem.objType   = 11
   else
      problem.objType   = 42
      problem.fun       = sopi_toScilabFunction(problem.objective, problem)
   end
endfunction
