// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function problem = sopi_min(fun, varargin)
   // Creates a minimisation problem.
   //
   // Calling Sequence
   //    problem = sopi_min(fun)
   //    problem = sopi_min(fun, c1, ..., cn)
   //
   // Parameters
   //    fun      : the objective function (sopiVar)
   //    ci       : constraints (sopiCst or list of sopiCst)
   //    problem  : the optimisation problem (sopiPro)
   //
   // See also
   //    sopi_var
   //    sopi_max
   //    sopi_solve
   //    sopi_example
   //
   if typeof(fun)~= 'sopiVar' then
      error(sprintf('The objective function must be a sopiVar, got a ''%s'' instead',typeof(fun)))
   end
   if ~isscalar(fun) then
      error('Multi-objective optimisation is not handled')
   end
   // Constraints & options
   options  = struct()
   cList    = list()
   if argn(2)>1 then
      // The options must be the last argument
      if typeof(varargin($)) == 'st' then
         options     = varargin($)
         varargin($) = null()
      end
      // Constraints come before
      for c = varargin
         if typeof(c) == 'sopiCst' then
            cList($+1) = c;
         elseif typeof(c) == 'list' & typeof(c(1))=='sopiCst' then
            cList = lstcat(cList, c)
         else
            error(sprintf('Unexpected input for the constraints: ''%s''',typeof(c)))
         end
      end
   end
   //
   problem = sopi_newProblem()
   problem = sopi_addAllVar(problem, fun, cList)
   problem = sopi_addObjective(problem, fun)
   problem = sopi_addConstraints(problem, cList)
   // --------------------------------------------------------------------------
   // Optimisation problem classification
   // --------------------------------------------------------------------------
   problem.class(1) = problem.objType
   if isempty(cList) then
      // The optimisation problem is unconstrained
      problem.class(2) = -1
   else
      // The optimisation problem is constrained
      if and(problem.cType == 0)
         // There are only bounds constraints
         problem.class(2) = 0
      elseif and(problem.cType <= 1)
         // Constraints are, at most, linear
         problem.class(2) = 1
      end
   end
   problem.solvers = sopi_getSolver(problem.class)
endfunction

function out = sopi_getSolver(class)
   out      = list()
   solvers  = sopi_listSolvers()
   for s = solvers
      if or(s.objClass == problem.class(1)) & or(s.cstClass == problem.class(2))
         out($+1) = s
      end
   end
endfunction
