// Author : Pierre Vuillemin (2016)
// License : GNU GPL

function [outVar, fopt, info] = sopi_solve(problem, varargin)
   // Solves an optimisation problem.
   //
   // Calling Sequence
   //    sopi_solve(problem)
   //    sopi_solve(problem, method)
   //    sopi_solve(problem, method, x0)
   //    sopi_solve(problem, method, x0, options)
   //
   // Parameters
   //    problem  : an optimisation problem (sopiPro)
   //    method   : the name of the optimisation method to use (string)
   //    x0       : the initialisation point (vector)
   //    options  : the options for the solver (structure)
   //
   // See also
   //    sopi_example
   //    sopi_var
   //    sopi_min
   //

   // --------------------------------------------------------------------------
   // Optional arguments handling
   // --------------------------------------------------------------------------
   method   = []
   x0       = []
   options  = []
   nv       = length(varargin)
   if nv >= 1 then
      method = varargin(1)
   end
   if nv >= 2 then
      x0    = varargin(2)
   end
   if nv == 3 then
      options = varargin(3)
   end
   // --------------------------------------------------------------------------
   // Solver
   // --------------------------------------------------------------------------
   if isempty(method) then
      solver = problem.solvers(problem.defaultSolver)
   else
      solverAvailable = %F
      for s = problem.solvers
         if s.name == method
            solverAvailable   = %T
            solver            = s
            break;
         end
      end
      if ~solverAvailable
         error('The method ''%s'' is not available problem of the type ''%s''',method,sopi_verboseProblemClass(problem.class))
      end
   end
   // --------------------------------------------------------------------------
   // Testing if the solver require x0
   // --------------------------------------------------------------------------
   if isempty(x0) & solver.requireX0 then
      sopi_print(-1,'The solver ''%s'' requires an initial point. Taking 0 as initial point.\n',solver.name)
      x0 = zeros(problem.nvar,1)
   end
   if ~isempty(x0) & ~solver.acceptX0 then
      sopi_print(-1,'The solver ''%s'' does not accept an initial point x0. Discarding the provided initial point.', solver.name)
   end
   //
   if isempty(options) then
      options = struct()
   end
   // --------------------------------------------------------------------------
   // Calling the solver
   // --------------------------------------------------------------------------
   sopi_checkUnavailableOpt(options, solver.possibleOptions, solver.name)
   tic()
   execstr('[xopt, fopt, info] = ' + solver.call + '(problem, x0, options, solver.possibleOptions)')
   info.elapsedTime  = toc()
   // --------------------------------------------------------------------------
   // Handling the outputs
   // --------------------------------------------------------------------------
   if ~isempty(xopt) then
      optVar = list()
      for v = problem.varList
         idx         = sopi_getVarIdxInProblem(problem, v)
         v.opt       = xopt(idx)
         optVar($+1) = v
      end
      outVar            = optVar
      // -----------------------------------------------------------------------
      // Assigning output optimal values
      // -----------------------------------------------------------------------
      clear v
      objective      = problem.objective
      objective.opt  = fopt
      optVar($+1)    = objective
      exVars         = sopi_getExistingSopiVar()
      couples        = list()
      for v = exVars
         for vopt = optVar
            if vopt.id == v(1).id
               couples($+1) = list(v(2), vopt)
            end
         end
      end
      out      = [];
      arg      = [];
      for i = 1:length(couples)
         out = [out,couples(i)(1)]
         arg = [arg,'couples(' + string(i) + ')(2)']
      end
      exStr = '[' + strcat(out,',') + '] = resume(' + strcat(arg,',')+');'
      execstr(exStr)
   else
      outVar   = []
      fopt     = []
   end

endfunction
